import { InjectionToken, Injectable, Inject, NgModule, Optional, SkipSelf } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { HttpErrorResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { of } from 'rxjs/observable/of';
import { _throw } from 'rxjs/observable/throw';
import { delay, retryWhen, switchMap, tap } from 'rxjs/operators';

var HTTP_REQUEST_RETRY_STRATEGIES = new InjectionToken('HTTP_REQUEST_RETRY_STRATEGIES');
var httpRetryFailuresSubject = new Subject();
var NgxHttpRetryService = /** @class */ (function () {
    function NgxHttpRetryService() {
        this.httpRetryFailures = httpRetryFailuresSubject.asObservable();
    }
    return NgxHttpRetryService;
}());
NgxHttpRetryService.decorators = [
    { type: Injectable },
];
NgxHttpRetryService.ctorParameters = function () { return []; };
function httpRequestRetry(retryStrategies) {
    var counts = {};
    function retryWhenNotifier(errors) {
        return errors.pipe(switchMap(function (error) {
            if (error instanceof HttpErrorResponse) {
                var retryStrategyIndex = retryStrategies.findIndex(function (rs) { return rs.statuses.includes(error.status); });
                var retryStrategy = retryStrategyIndex > -1 ? retryStrategies[retryStrategyIndex] : undefined;
                counts[retryStrategyIndex] = (counts[retryStrategyIndex] || 0) + 1;
                var shouldRetry = retryStrategy && counts[retryStrategyIndex] < retryStrategy.maxCount;
                var retryDelay = retryStrategy && retryStrategy.delayFn ? retryStrategy.delayFn(counts[retryStrategyIndex]) : 0;
                if (!shouldRetry && retryStrategy) {
                    httpRetryFailuresSubject.next(error);
                    if (retryStrategy.onFailure) {
                        retryStrategy.onFailure(error);
                    }
                }
                return shouldRetry ? of(error).pipe(delay(retryDelay)) : _throw(error);
            }
        }));
    }
    return function (source) { return source.pipe(retryWhen(retryWhenNotifier)); };
}
var attemptNumberHeader = 'X-Request-Attempt-Number';
var NgxHttpRetryInterceptor = /** @class */ (function () {
    function NgxHttpRetryInterceptor(retryStrategies) {
        this.retryStrategies = retryStrategies;
    }
    NgxHttpRetryInterceptor.prototype.intercept = function (request, next) {
        var result;
        if (this.retryStrategies.length && request.method.toUpperCase() === 'GET') {
            var attemptNumber_1 = 0;
            result = of(undefined).pipe(tap(function () {
                attemptNumber_1++;
            }), switchMap(function () { return next.handle(getRequestWithAttemptNumber(request, attemptNumber_1)); }), httpRequestRetry(this.retryStrategies));
        }
        else {
            result = next.handle(request);
        }
        return result;
    };
    return NgxHttpRetryInterceptor;
}());
NgxHttpRetryInterceptor.decorators = [
    { type: Injectable },
];
NgxHttpRetryInterceptor.ctorParameters = function () { return [
    { type: Array, decorators: [{ type: Inject, args: [HTTP_REQUEST_RETRY_STRATEGIES,] },] },
]; };
var ngxHttpRetryInterceptorProvider = {
    provide: HTTP_INTERCEPTORS,
    useClass: NgxHttpRetryInterceptor,
    multi: true
};
function getRequestWithAttemptNumber(request, attemptNumber) {
    return request.clone({ headers: request.headers.append(attemptNumberHeader, attemptNumber.toString()) });
}
var NgxHttpRetryModule = /** @class */ (function () {
    function NgxHttpRetryModule(parentModule) {
        if (parentModule) {
            throw new Error('NgxHttpRetryModule is already loaded. Import it only once (e.g. in your AppModule or CoreModule).');
        }
    }
    NgxHttpRetryModule.forRoot = function () {
        return {
            ngModule: NgxHttpRetryModule,
            providers: [NgxHttpRetryService, ngxHttpRetryInterceptorProvider]
        };
    };
    return NgxHttpRetryModule;
}());
NgxHttpRetryModule.decorators = [
    { type: NgModule, args: [{
                imports: [HttpClientModule]
            },] },
];
NgxHttpRetryModule.ctorParameters = function () { return [
    { type: NgxHttpRetryModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
]; };

export { HTTP_REQUEST_RETRY_STRATEGIES, httpRequestRetry, NgxHttpRetryService, NgxHttpRetryModule, NgxHttpRetryInterceptor as ɵa, ngxHttpRetryInterceptorProvider as ɵb };
//# sourceMappingURL=ngx-http-retry.js.map
